#ifndef AUTOLINK_H_
#define AUTOLINK_H_

#include <string>
#include <vector>
#include <map>

#include <rtm/RTObject.h>


#define CALLBACK_TYPE_COMP		0x00000001
#define CALLBACK_TYPE_ZOMB		0x00000002
#define CALLBACK_TYPE_COMP_ALL	0x00000003

#define CALLBACK_TYPE_CNTX		0x00000004
#define CALLBACK_TYPE_SERV		0x00000008
#define CALLBACK_TYPE_SERV_ALL	0x0000000C

#define CALLBACK_TYPE_ALL		(CALLBACK_TYPE_COMP | CALLBACK_TYPE_ZOMB | CALLBACK_TYPE_SERV | CALLBACK_TYPE_CNTX)

enum _port_type{
	in_port = 1,
	out_port,
	corba_port
};

enum controlType{
	cont_type_activate_comp = 1,
	cont_type_deactivate_comp,
	cont_type_start_comp,
	cont_type_stop_comp,
	cont_type_reset_comp,
	cont_type_exit_comp
};

struct ComponentData{
	std::string	name;
	std::string path;
	CORBA::Object_ptr pPrt;
};

typedef void (*EnumCallbackProc)(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pComp);

typedef std::map< std::string, RTC::CorbaNaming>	corbaNamingList;
typedef corbaNamingList::value_type corba_naming_list;



/* NamingServer関連 */
int					connectNamingServer(const std::string serverName);
std::string			getNamingServerName(std::string namingServer);

CORBA::Object_ptr	findComponent(const std::string compName, const CORBA::Object_ptr obj = NULL, const bool showCandidateComp = true);
int					componentControl(std::string compName, controlType contID, CORBA::Object_ptr pComponent = NULL);
int					linkComponent(std::string srcComp, std::string srcPort, std::string destComp, std::string destPort,
									std::string name = "", std::string if_type = "corba_cdr", std::string df_type = "push", std::string ss_type = "flush");

RTC::PortService_ptr	getPortInfo(std::string compName, std::string portName);
bool					isConnected(RTC::PortService_ptr src, RTC::PortService_ptr dest);
std::string				getPortProperty(RTC::PortService_ptr src, std::string propName);
std::string				getPortType(RTC::PortService_ptr pPort);
std::string				getPortDataType(RTC::PortService_ptr pPort);
std::string				getRegisterNamingName(CORBA::Object_ptr pComp);

void enumNamingServers(const EnumCallbackProc func, const short callbackType = CALLBACK_TYPE_ALL, const std::string serverName = "");
void enumCosNaming(const EnumCallbackProc func, const short callbackType, const std::string namingName, RTC::CorbaNaming curNaming, CosNaming::NamingContext_ptr curContext, int layered = 0);

void findComponentCallback(std::string name, std::string kind, int type, int layered = 0, std::string path = "", CORBA::Object_ptr pComp = NULL);


#ifdef _RTC_CONTROL_MAIN
/* CorbaORB情報 */
CORBA::ORB_var				corbaORB;
/* RTCネームサーバ情報文字列 */
corbaNamingList				namingServerList;
std::string					findCompName;
std::string					findCompPath;
std::vector< ComponentData > component_list;
#else
/* CorbaORB情報 */
extern CORBA::ORB_var				corbaORB;
/* RTCネームサーバ情報文字列 */
extern corbaNamingList				namingServerList;
extern std::string					findCompName;
extern std::string					findCompPath;
extern std::vector< ComponentData > component_list;
#endif

#endif /*AUTOLINK_H_*/
