#include "ConvertXML.h"
#include <iconv.h>

ConvertGrammar::ConvertGrammar()/* : TextDataFile()*/
{
#ifdef USE_XERCESC
	try {
		XMLPlatformUtils::Initialize();
	} catch(...) {
		std::cout << "Xerces-C++の初期化に失敗しました。" << std::endl;
	}

	namesParser		= new XercesDOMParser();

	if(namesParser != 0){
		namesParser->setValidationScheme(XercesDOMParser::Val_Always);
		namesParser->setDoNamespaces(true);
	}

	grammarParser	= new XercesDOMParser();
	if(grammarParser != 0){
		grammarParser->setValidationScheme(XercesDOMParser::Val_Always);
		grammarParser->setDoNamespaces(true);
	}
#else
	dataPos			= 0;
	readDataPos		= 0;
	xmlName			= "";
	initMemberDummy	= 0;
	memset(dataBuff, 0, sizeof(dataBuff));
#endif

	// 基底クラスが保持しているメンバにデータを設定
	{
		FieldData	fieldInfo;

		fieldInfo.modFlg		= true;
		fieldInfo.addFlg		= true;
		fieldInfo.rangeFilter	= false;
		fieldInfo.fieldType		= ftString;
		fieldInfo.fieldName		= "grapheme";
		fieldInfo.fieldName_org	= "grapheme";
		fieldNameList.push_back(fieldInfo);

		fieldInfo.fieldName		= "phoneme";
		fieldInfo.fieldName_org	= "phoneme";
		fieldNameList.push_back(fieldInfo);
	}

}

ConvertGrammar::~ConvertGrammar()
{
#ifdef USE_XERCESC
	delete namesParser;
	delete grammarParser;
#endif
}


/**
 * @brief 音声認識辞書を開く
 * @ret 辞書を開いた結果
 * 		true : 正常に開いた, false : 開くことに失敗した
 * @note 正確には音声認識辞書を検索した結果を返す。
 */
bool ConvertGrammar::openGrammar(std::string name)
{
	bool	result = true;

	/*
	 * 認識辞書のフルパスを組み上げる
	 * グローバル設定ファイルから、音声認識辞書ファイルの位置を取得する
	 * 指定された「name」に該当するファイルを探す(オープンを試す？)
	 * 存在すれば良し。
	 *
	 * @note grammarファイルも探して、両方が在った場合のみOKとする。
	 */

	std::string	dataFolder, xmlFileName;
	TS_Property	systemProperty;

	xmlName = name;
	/*
	 * 2010.12.22 データファイルのファイル名はプロパティファイルで定義するように変更
	 * ただし、複数ファイルがある場合は、フォルダまでしか指定していない。
	 * 最終文字が'/'の場合は、フォルダ指定であり、ターゲットファイルが複数ある事を意味する。
		// データファイルを特定する
		dataFile = query_info->GetQueryPair("File");
	*/
	dataFolder = systemProperty[VOICE_RECOGNITION];
	if(dataFolder.length() == 0){
		std::cout << "開くファイル 指定されてない" << name << std::endl;
		return false;
	}

	if(FindXmlFile(dataFolder + XML_FILE_NAME, dataFolder + GRAMMAR_NAME) == false){
		std::cout << "開くファイル がなかった" << name << "<br />"<< std::endl;
		return false;
	}

	xmlName = dataFolder + xmlName;

#ifdef USE_XERCESC
	namesParser->parse(XML_FILE_NAME.c_str());
	grammarParser->parse(GRAMMAR_NAME.c_str());

	// ドキュメントツリーから、データを取得する
	// ドキュメントオブジェクトを取得
	DOMDocument* doc = namesParser->getDocument();
	// ルートノードを取得する
	DOMElement* root = doc->getDocumentElement();

	// データ追加ノードを探す
	DOMNodeList	*nodeList = root->getElementsByTagName(XMLString::transcode("grapheme"));
	DOMNodeList	*kanaList = root->getElementsByTagName(XMLString::transcode("phoneme"));

	if(nodeList == 0)
		std::cout << "ノードの数は 0 " << std::endl;
	if(kanaList == 0)
		std::cout << "かなの数は 0 " << std::endl;

	// フィールド一覧作成
	for(unsigned int cnt = 0; cnt < nodeList->getLength(); cnt++){
		std::string	tagName, value;
		field_data	oneRecord;
		DOMNodeList	*targetnodeList;
		DOMNode *nodeData, *parentNode, *nodeValue;

		// 親ノードに移動する
		parentNode = nodeList->item(cnt)->getParentNode();

		if(parentNode == 0)
			continue;

		targetnodeList = parentNode->getChildNodes();
		if(targetnodeList == 0)
			continue;

		for(nodeData = parentNode->getFirstChild(); nodeData != 0; nodeData = nodeData->getNextSibling()){
			if(nodeData->getNodeType() != DOMNode::ELEMENT_NODE)
				continue;

			// タグ判別
			tagName = xml_to_string(nodeData->getNodeName());
			if((tagName.compare("phoneme") != 0) && (tagName.compare("grapheme") != 0))
				continue;

			// 要素文字列取得
			nodeValue = nodeData->getFirstChild();
			if(nodeValue != 0){
				// データ追加
				value = xml_to_string(nodeValue->getNodeValue());
				if(oneRecord.find(tagName) == oneRecord.end()){
					oneRecord.insert(field_pair(tagName, value));
				}
				else{
					oneRecord[tagName] = oneRecord[tagName] + ", ";
					oneRecord[tagName] = oneRecord[tagName] + value;
				}
			}
		}
		// データ収納
		recordBuff.insert(record_pair(oneRecord["grapheme"], oneRecord));
	}

#else
	// かなーり強引にデータを取得してくる
#endif

	return result;
}

bool ConvertGrammar::removeRecord(const std::string key)
{

	delData(recordBuff[key]["phoneme"]);

	delData(key);

	// バッファデータも削除する
	return TextDataFile::removeRecord(key);
}

bool ConvertGrammar::writeRecord(const std::string key_val, const field_value recordData)
{
	std::string	key = key_val;
	// データの存在確認を行い、処理を振り分ける
	if(key.length() == 0)
		key = recordData[0];

	if(FindData(key) != (off_t)(-1)){
		// データあり
		if(key.compare(recordData[0]) == 0){
//			changeData(key, recordData[0]);
//			changeData(recordBuff[key]["phoneme"], recordData[1]);
			changeData(recordData[0], recordData[1]);
		}
		else{
			// 元データを削除する必要がある
			removeRecord(key);
			addData(recordData[0], recordData[1]);
		}
	}
	else{
		// データなし
		addData(recordData[0], recordData[1]);
	}
	return true;
}

bool ConvertGrammar::isData(std::string val)
{
	return (FindData(val) > 0);
}

bool ConvertGrammar::changeData(std::string src, std::string dest)
{
#ifndef USE_XERCESC
	bool	checkStop = false;
#endif
	bool	result = true;

	if(dest.length() == 0)
		return false;

	// ファイルの存在を確認する
	if( FindXmlFile(XML_FILE_NAME, GRAMMAR_NAME) == true){
#ifdef USE_XERCESC
#if 0
		// ドキュメントオブジェクトを取得
		DOMDocument* doc = namesParser->getDocument();
		// ルートノードを取得する
		DOMElement* root = doc->getDocumentElement();

		// 変更ノードを探す
		DOMNode	*targetNode	= FindData(root, src);
//		DOMNode	*parentNode = root->getElementsByTagName("lexicon");

		if(targetNode != 0){
			targetNode->setNodeValue(XMLString::transcode(dest.c_str()));
			writeXML();
		}
		doc = grammarParser->getDocument();
		root = doc->getDocumentElement();
		targetNode	= FindData(root, src);
		if(targetNode != 0){
			targetNode->setNodeValue(XMLString::transcode(dest.c_str()));
			writeXML();
		}
#else
		removeRecord(src);
		addData(src, dest);
#endif
#else
		// データの読み込みを行う
		int		fd;
		int		/*readLen, }*/writeTemp, findData;
		off_t	curPos = 0, wrtPos;
/*		char	temp[SIZE_1K];*/

		// 最初のターゲットファイルを開く
		fd = open(XML_FILE_NAME.c_str(), O_RDWR);
		while(fd != -1){
			// 該当データが存在しているか確認する
			findData = FindData(fd, src);

			writeTemp = 0;

			if(findData > 0){
				// 該当データらしきデータが存在してるので、データを変換して出力する

				// 元データとデータ長が違わなければ簡単だが・・・( ※文字コードはUTF-8を採用しているので、文字によりMB長が変わる事はないはず )
				{
					// まず、データ部までの出力を行う
					lseek(fd, readDataPos, SEEK_SET);
					write(fd, dataBuff, findData);
					curPos = lseek(fd, 0, SEEK_CUR);

					// データ部後続データを出力する
					wrtPos = fMove(fd, dataPos + strlen(src.c_str()), dataPos + strlen(dest.c_str()));

					// 変換後データ部を出力する
					lseek(fd, curPos, SEEK_SET);
					write(fd, dest.c_str(), strlen(dest.c_str()));
				}
			}

			curPos = lseek(fd, wrtPos, SEEK_SET);
			ftruncate(fd, wrtPos);
			close(fd);

			if(checkStop)
				break;

			checkStop = true;
			fd = open(GRAMMAR_NAME.c_str(), O_RDWR);
		}
		close(fd);
#endif
	}

	return result;
}

bool ConvertGrammar::delData(std::string val)
{
#ifdef USE_XERCESC
	DOMNode	*parentNode;
	// ドキュメントオブジェクトを取得
	DOMDocument* doc = namesParser->getDocument();
	// ルートノードを取得する
	DOMElement* root = doc->getDocumentElement();

	// 変更ノードを探す
	DOMNode	*targetNode	= FindData(root, val);
	if(targetNode != 0){
		parentNode = targetNode->getParentNode();

		targetNode = targetNode->getParentNode();
		parentNode = parentNode->getParentNode();
		targetNode = targetNode->getParentNode();
		parentNode = parentNode->getParentNode();
		if((targetNode != 0) && (parentNode != 0)){
			parentNode->removeChild(targetNode);
			writeXML();
		}
	}

	doc = grammarParser->getDocument();
	root = doc->getDocumentElement();
	targetNode	= FindData(root, val);
	if(targetNode != 0){
		parentNode = targetNode->getParentNode();
		if((targetNode != 0) && (parentNode != 0)){
			targetNode = targetNode->getParentNode();
			parentNode = parentNode->getParentNode();
			parentNode->removeChild(targetNode);
			writeXML();
		}
	}
#else
	bool	result;
	std::string	findKey;

	findKey = "</";
	findKey = findKey + XML_TAG_LEXEME;
	result = delData(XML_FILE_NAME, findKey, val);

	findKey = "</";
	findKey = findKey + XML_TAG_ITEM;
	result = delData(GRAMMAR_NAME, findKey, val);
#endif
	return true;
}

bool ConvertGrammar::delData(std::string xmlFile, std::string findKey, std::string val)
{
#ifndef USE_XERCESC
	int		fd;
	int		keyPos, findData;
	off_t	orgPos, curPos;

	// 該当データが出るまで、ある規則にそってファイルポインタを移動させる
	// 規則とは・・・"</lexeme>" or "</item>"単位でファイルポインタを区切る事だ

	// 該当データを探す
	fd = open(xmlFile.c_str(), O_RDWR);
	if(fd != -1){
		// 該当データが存在しているか確認する
		findData = FindData(fd, findKey);
		if(findData > 0){
			// 該当データあり
			// 現在のファイルポインタを取得
			orgPos = lseek(fd, 0, SEEK_CUR);
			// ファイルポインタをバッファデータ出力位置に変更
			lseek(fd, readDataPos, SEEK_SET);
			// 該当データ近くのタグを探す
			keyPos = findKeyTag(findKey, dataBuff, 0, findData);
			if(keyPos != -1){
				keyPos = keyPos + findKey.length();
				// キー終端までを出力
				write(fd, dataBuff, keyPos);
			}
			// キー後方データ位置を探す
			keyPos = findKeyTag(findKey, dataBuff, findData, 0);
			if(keyPos != -1){
				keyPos = keyPos + findKey.length();
				// 終端までを出力
				write(fd, &(dataBuff[keyPos]), strlen(&(dataBuff[keyPos])));

				// 後続データ書き込み位置取得
				curPos = lseek(fd, 0, SEEK_CUR);
			}

			// 残データを全て移動してくる
			for(int readSize = SIZE_2K; readSize < SIZE_2K;){
				lseek(fd, orgPos, SEEK_SET);
				readSize = read(fd, dataBuff, SIZE_2K);
				lseek(fd, curPos, SEEK_SET);
				write(fd, dataBuff, readSize);
				orgPos = orgPos + readSize;
				curPos = curPos + readSize;
			}
			close(fd);
		}
	}
#endif
	return true;
}

int ConvertGrammar::findKeyTag(std::string key, char* buff, int searchStart, int searchEnd)
{
	char	*findPos;
	int		result = -1;
	int		searchMax;

	searchMax = searchEnd != 0 ? searchEnd : strlen(buff);
	for(int curPos = searchStart; (int)(curPos + key.length()) < searchMax; curPos++){
		findPos = strstr(&(buff[curPos]), key.c_str());
		if(findPos != 0){
			// タグが見つかった
			result = (int)(findPos - buff);
			if(result < searchEnd)
				curPos = result;
			else{
				result = -1;
				break;
			}
		}
	}

	return result;
}



bool ConvertGrammar::addData(std::string val, std::string kana)
{
#ifdef USE_XERCESC
	// ドキュメントオブジェクトを取得
	DOMDocument* doc = namesParser->getDocument();
	// ルートノードを取得する
	DOMElement* root = doc->getDocumentElement();

	// データ追加ノードを探す
	DOMNodeList	*addNodeParent = root->getElementsByTagName(XMLString::transcode("lexicon"));


	if(root != 0){
		DOMElement	*addNode		= doc->createElementNS(root->getAttribute(XMLString::transcode("xmlns")), XMLString::transcode("lexeme"));
		DOMElement	*graphemeNode	= doc->createElementNS(root->getAttribute(XMLString::transcode("xmlns")), XMLString::transcode("grapheme"));
//		DOMElement	*phonemeNode	= doc->createElementNS(root->getAttribute(XMLString::transcode("xmlns")), XMLString::transcode("phoneme"));
		DOMText		*keyData		= doc->createTextNode(XMLString::transcode(val.c_str()));
//		DOMText		*kanaData		= doc->createTextNode(XMLString::transcode(kana.c_str()));
		grammar_item_list	kanaList;

		if(doc == 0){
			std::cout << "DOMDocument create error! " << std::endl;
			return false;
		}
		if(addNode == 0){
			std::cout << "lexeme node create error!" << std::endl;
			return false;
		}

		if(graphemeNode == 0){
			std::cout << "grapheme node create error!" << std::endl;
			return false;
		}

		graphemeNode->appendChild((DOMNode*)keyData);
		addNode->appendChild(graphemeNode);


		kanaList = string_token(kana, " ,");
		for(grammar_item_list::iterator it = kanaList.begin(); it != kanaList.end(); it++){
			DOMElement	*phonemeNode	= doc->createElementNS(root->getAttribute(XMLString::transcode("xmlns")), XMLString::transcode("phoneme"));
			DOMText		*kanaData		= doc->createTextNode(XMLString::transcode(it->c_str()));

			if(phonemeNode == 0){
				std::cout << "phoneme node create error!" << std::endl;
				continue;
			}
			phonemeNode->appendChild((DOMNode*)kanaData);
			addNode->appendChild(phonemeNode);
		}

		// ノードを追加する
		root->appendChild(addNode);
		writeXML();
	}

	doc = grammarParser->getDocument();
	root = doc->getDocumentElement();
	addNodeParent = root->getElementsByTagName(XMLString::transcode("one-of"));
	if(addNodeParent != 0){
		/*
		DOMElement	*addNode		= doc->createElement(XMLString::transcode("item"));
		*/
		DOMElement	*addNode		= doc->createElementNS(root->getAttribute(XMLString::transcode("xmlns")), XMLString::transcode("item"));
		DOMText		*keyData		= doc->createTextNode(XMLString::transcode(val.c_str()));

//		addNode->setNodeValue(XMLString::transcode(val.c_str()));
		// ノードを追加する
		addNode->appendChild((DOMNode*)keyData);
		addNodeParent->item(0)->appendChild(addNode);
		writeXML();
	}

#else
	{
		field_data	oneRecord;
		// データ追加

		// データ収納
		oneRecord.insert(field_pair("grapheme", val));
		oneRecord.insert(field_pair("phoneme", kana));

		recordBuff.insert(record_pair(val, oneRecord));
	}
	addData(XML_FILE_NAME, val, kana);
	addData(GRAMMAR_NAME, val, "");
#endif
	return true;
}
bool ConvertGrammar::addData(std::string xmlFile, std::string val, std::string kana)
{
	bool			result = true;
#ifdef USE_XERCESC

#else
	int				fd;
	int				keyPos, readLen, findData;
	bool			result = true;
	std::string		kanjiVal, kanaVal, writeVal;

	// 該当データを探す
	fd = open(xmlFile.c_str(), O_RDWR);
	if(fd != -1){
		// 該当データが存在しているか確認する
		findData = FindData(fd, val);
		if(findData != -1){
			std::string	findKey;

			findKey = "<";
			findKey = findKey + XML_TAG_LEXEME;

			// ファイルの先頭から再度読み込む
			memset(dataBuff, 0, sizeof(dataBuff));
			lseek(fd, 0, SEEK_SET);
			readLen = read(fd, dataBuff, SIZE_2K);
			// タグを探す
			keyPos = findKeyTag(findKey, dataBuff, 0, 0);
			if(keyPos != -1){
				// データを追加する？
				if(kana.length() > 0){
					// namesの方
					/*
					 * <lexeme>
					 * 	<grapheme>津幡</grapheme>
					 * 	<phoneme>{{KANA|つばた}}</phoneme>
					 * </lexeme>
					 *
					 * を追加
					 */
					kanjiVal	= "<grapheme>" + val + "</grapheme>\n";
					kanaVal		= "<phoneme>{{KANA|" + kana + "}}</phoneme>\n";
					writeVal	= "\n<lexeme>" + kanjiVal + kanaVal + "</lexeme>";
				}
				else{
					// grammarの方
					/*
					 * <item>津幡</item>
					 *
					 * を追加
					 */
					writeVal = "\n<item>" + val + "</item>";
				}
				fMove(fd, keyPos, keyPos + strlen(writeVal.c_str()));
				lseek(fd, keyPos, SEEK_SET);
				write(fd, writeVal.c_str(), strlen(writeVal.c_str()));
			}
		}
		else{
			return false;
		}
	}
#endif
	return result;
}

#ifdef USE_XERCESC
DOMNode* ConvertGrammar::FindData(DOMNode *node, std::string val)
{
	std::string	valStr;
	DOMNode	*result = 0;
//	XMLCh	*nodeVal;
	DOMNode *nextNode = node;

	if(node == 0)
		return 0;

	// ノードの値を確認する
//	nodeVal = node->getNodeValue();
	valStr = xml_to_string(node->getNodeValue());

	if(valStr.compare(val) == 0){
		return node;
	}

	// 兄弟ノード
	do{
		result = FindData(nextNode->getFirstChild(), val);
		nextNode = nextNode->getNextSibling();
	}while((nextNode != 0) && (result == 0));

	return result;
}
#endif

off_t ConvertGrammar::FindData(std::string val)
{
	off_t		result = -1;

	/*
	 * まずは、ファイルが存在しているか確認する
	 */
	if( FindXmlFile(XML_FILE_NAME, GRAMMAR_NAME) == true){
#ifdef USE_XERCESC
		// ドキュメントオブジェクトを取得
		DOMDocument* doc = namesParser->getDocument();
		// ルートノードを取得する
		DOMElement* root = doc->getDocumentElement();
		// 再帰的にXMLツリーを降っていく
		if(FindData(root, val) != 0)
			result = 1;
		return result;
#else
		// データの読み込みを行う
		int		fd;

		fd = open(XML_FILE_NAME.c_str(), O_RDWR);
		// 該当データが存在しているか確認する
		result = FindData(fd, val);
		close(fd);
		if(result < 0){
			fd = open(GRAMMAR_NAME.c_str(), O_RDWR);
			result = FindData(fd, val);
			close(fd);
		}
#endif
	}

	return result;
}

/**
 * bbrief 音声認識辞書の存在を確認する
 *
 */
bool ConvertGrammar::FindXmlFile(std::string xmlFileName, std::string gramamrName)
{
	if(access(xmlFileName.c_str(), F_OK | R_OK | W_OK) != 0){
		return false;
	}

	if(access(gramamrName.c_str(), F_OK | R_OK | W_OK) != 0){
		return false;
	}

	return true;
}

/**
 * @brief 指定されたファイルディスクリプタのデータを読み込み、文字列検索を行う
 *
 * @ret > 0 : 文字列あり, 0 : 文字列なし, -1 : ファイル終端に到達
 */
off_t ConvertGrammar::FindData(int fd, std::string val)
{
	int	result = 0;
#ifndef USE_XERCESC
	int	readLen;
	int	readSize = SIZE_2K;
	char	*findStr;

	while(1){
		// 最初は２k読み込み
		readLen = read(fd, &(dataBuff[(SIZE_2K - readSize)]), readSize);

		// 該当文字列を検索
		findStr = strstr(dataBuff, val.c_str());
		if( findStr != 0){
			result = (int)(findStr) - (int)(dataBuff);
			break;
		}
		if(readSize > readLen){
			result = -1;
			break;
		}
		dataPos = dataPos + readSize;
		readSize = SIZE_1K;
		memmove(dataBuff, &(dataBuff[readSize]), readSize);
		memset(&(dataBuff[readSize]), 0, readSize);
		readDataPos = readDataPos + readSize;
	}
	dataPos = dataPos + result;

	result = (dataPos - readDataPos);
#endif
	return result;
}

#ifdef USE_XERCESC
int ConvertGrammar::writeXML()
{
	// DOMWriterオブジェクト(シリアライザー)を生成
	DOMImplementation	*impl			= DOMImplementationRegistry::getDOMImplementation(XMLString::transcode("LS"));
//	DOMWriter			*theSerializer	= ((DOMImplementationLS*)impl)->createDOMWriter();
	DOMLSSerializer		*theSerializer	= ((DOMImplementationLS*)impl)->createLSSerializer();
	DOMLSOutput			*outputTarget	= ((DOMImplementationLS*)impl)->createLSOutput();

	// ドキュメントオブジェクトを取得
	DOMDocument	*doc = namesParser->getDocument();
	// ルートノードを取得する
	DOMElement	*root = doc->getDocumentElement();

	// 出力時のエンコーディングを設定
	theSerializer->getDomConfig()->setParameter(XMLString::transcode("format-pretty-print"), true);
//	theSerializer->setEncoding(XMLString::transcode("UTF-8"));

	// XMLファイルに書き出す
	XMLFormatTarget	*fileFormatTarget = new LocalFileFormatTarget(XML_FILE_NAME.c_str());
	outputTarget->setEncoding(XMLString::transcode("UTF-8"));

	outputTarget->setByteStream(fileFormatTarget);

//	theSerializer->writeNode(fileFormatTarget, *doc);
	theSerializer->write(root, outputTarget);

	delete fileFormatTarget;

	doc = grammarParser->getDocument();

	root = doc->getDocumentElement();

	fileFormatTarget = new LocalFileFormatTarget(GRAMMAR_NAME.c_str());

	outputTarget->setByteStream(fileFormatTarget);
//	theSerializer->writeNode(fileFormatTarget, *doc);

	theSerializer->write(root, outputTarget);

	delete fileFormatTarget;

	return true;
}

std::string xml_to_string(const XMLCh* xml)
{
	size_t	utf16_len, utf8_len;
	const XMLCh	*utf16 = xml;
	char	*utf8, *start;
	std::string	result = "";
	iconv_t iconv_id;

	if(xml == 0)
		return result;

	iconv_id = iconv_open("utf-8", "utf-16");
	if(iconv_id != (iconv_t)(-1)){
		utf16_len = 0;
		for(const XMLCh* temp = utf16; *temp != 0; temp++)
			utf16_len++;
		utf16_len = utf16_len * 2;
		utf8_len = utf16_len * 3;
		utf8 = (char*)(malloc(utf8_len));
		start = utf8;
		memset(utf8, 0, utf8_len);
		iconv(iconv_id, (char**)(&utf16), &utf16_len, &utf8, &utf8_len);
		iconv_close(iconv_id);
		result = start;
		free(start);
	}
/*
	std::stringstream ss;

	ss << xml;
	return ss.str();
*/
	return result;
}

#endif
