#ifndef CONVERT_XML_H_
#define CONVERT_XML_H_

#define	USE_XERCESC	1

#ifdef USE_XERCESC
#include <xercesc/framework/XMLFormatter.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
//#include <xercesc/dom/DOMWriter.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMLSSerializer.hpp>
#include <xercesc/dom/DOMLSOutput.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>

#if defined(XERCES_NEW_IOSTREAMS)
#include <iostream>
#else
#include <iostream.h>
#endif

#else
#include <iostream>
#endif		// USE_XERCESC

#include <sstream>
#include <strstream>


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string>
#include <vector>

#include "Tools.h"
#include "TextData.h"
#include "TSProperty.h"

#ifdef USE_XERCESC
XERCES_CPP_NAMESPACE_USE

std::string		xml_to_string(const XMLCh* xml);

#endif

/*
 * --- names ---
 *	<lexeme>
 *		<grapheme>二宮</grapheme>
 *		<phoneme>{{KANA|にのみや}}</phoneme>
 *	</lexeme>
 *	<lexeme>
 *		<grapheme>津幡</grapheme>
 *		<phoneme>{{KANA|つばた}}</phoneme>
 *	</lexeme>
 * --------------
 *
 * --- grammar ---
 *	<one-of>
 *		<item>二宮</item>
 *		<item>津幡</item>
 *	</one-of>
 * --------------
 */


#define	XML_TAG_LEXEME		"lexeme>"
#define	XML_TAG_GRAPHEME	"grapheme>"
#define	XML_TAG_PHONEME		"phoneme>"

#define	XML_TAG_ONEOF		"one-of>"
#define	XML_TAG_ITEM		"item>"


#define	XML_FILE_NAME	(xmlName + ".xml")
#define	GRAMMAR_NAME	(xmlName + "-grammar.xml")

#define	SIZE_1K			1024
#define	SIZE_2K			2048


typedef std::vector< std::string >		grammar_item_list;
typedef grammar_item_list::iterator	grammer_it;

class ConvertGrammar : public TextDataFile{
public:
	ConvertGrammar();
	~ConvertGrammar();

public:
	bool openGrammar(std::string name);
	bool isData(std::string val);
	bool changeData(std::string src, std::string dest);
	bool delData(std::string val);
	bool addData(std::string val, std::string kana);

	// メソッドオーバーライド
	bool	openFile(const std::string fileName = "")		{	return openGrammar(fileName);	}
	int		readData()										{	return true;	}
	bool	isAdd(std::string	fieldName)					{	return true;	}
	bool	isModify(std::string	fieldName)				{	return true;	}
	bool	isRange(std::string	fieldName)				{	return false;	}
	bool	isDate(std::string	fieldName)					{	return false;	}
	bool	isTime(std::string	fieldName)					{	return false;	}
	int		writeData(const std::string fileName = "")	{	return writeXML();	}
	//	record_it	getTopRecord();
	//	record_it	getNextRecord();
	//	field_value	getFields();
	//	field_value	getRecord();
	bool	removeRecord(const std::string key);
	bool	writeRecord(const std::string key, const field_value recordData);

private:
	bool		delData(std::string xmlFile, std::string findKey, std::string val);
	bool		addData(std::string xmlFile, std::string val, std::string kana);
	bool		FindXmlFile(std::string xmlFileName, std::string gramamrName);
	off_t		FindData(int fd, std::string val);
	off_t		FindData(std::string val);
	int			findKeyTag(std::string key, char* buff, int searchStart, int searchEnd);
#ifdef USE_XERCESC
	int			writeXML();
	DOMNode*	FindData(DOMNode *node, std::string val);
#endif

private:
	std::string			xmlName;

#ifdef USE_XERCESC
	XercesDOMParser	*namesParser;
	XercesDOMParser	*grammarParser;
#else
	off_t			dataPos;
	off_t			readDataPos;
	int				initMemberDummy;
	char			dataBuff[2050];
#endif
};

#endif
