#include <algorithm>
#include <functional>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include "Tools.h"
#include "TSProperty.h"
#include "SystemControl.h"

#define GRAMMAR_COMBINATION	1

#ifdef _DEBUG
extern FILE		*pDbgLog;
#endif

SystemControl::SystemControl(QueryInfo *query) : CGI_Engine(query)
{
	// 自身が管理するHTMLページのマップを作る
	htmlList.insert(string_pair("default", "./cgi-bin/SystemState.html"));
	htmlList.insert(string_pair("ListView", "./cgi-bin/TableRecord.html"));
	htmlList.insert(string_pair("ModData", "./cgi-bin/DataEditor.html"));
	htmlList.insert(string_pair("UpdateData", "./cgi-bin/DataUpdate.html"));
	htmlList.insert(string_pair("DelData", "./cgi-bin/DataUpdate.html"));
	htmlList.insert(string_pair("FilterData", "./cgi-bin/DataFilter.html"));
	htmlList.insert(string_pair("GrammarSelect", "./cgi-bin/SpeechGrammarList.html"));

	curData.insert(string_pair("FieldHeaders", ""));
	curData.insert(string_pair("FieldValues", ""));
	curData.insert(string_pair("FolderKey", ""));
	curData.insert(string_pair("FileName", ""));
	curData.insert(string_pair("KeyValue", ""));
	curData.insert(string_pair("DataKey", ""));
	curData.insert(string_pair("UpdateResult", ""));
	curData.insert(string_pair("Attribute", ""));
	curData.insert(string_pair("FunctionBar", ""));

	textData = 0;

}

SystemControl::~SystemControl()
{
	if(textData != 0)
		delete textData;
}

bool SystemControl::readDataFile()
{
	std::string dataFullPath;
	std::string	dataFolder;

	if(query_info->GetSubFunctionName() == "default")		return true;
	if(query_info->GetSubFunctionName() == "GrammarSelect")	return true;

	folderKey = query_info->GetMode();
	if(folderKey.length() == 0)	dataFolder = "default";

	{
		TS_Property	systemProperty;

		curData["KeyValue"]		= query_info->GetQueryPair("KeyValue");

		if(folderKey.compare("VoiceRecognition") != 0){
//			dataFolder = systemProperty.getValue(dataFolder);
			dataFolder = systemProperty[folderKey];
			if(dataFolder.length() == 0) dataFolder = "/tmp/";
//			if(dataFolder[dataFolder.length()-1] != '/')	dataFolder = dataFolder + "/";
/*
 * 2010.12.22 データファイルのファイル名はプロパティファイルで定義するように変更
 * ただし、複数ファイルがある場合は、フォルダまでしか指定していない。
 * 最終文字が'/'の場合は、フォルダ指定であり、ターゲットファイルが複数ある事を意味する。
		// データファイルを特定する
*/
			// もしかしたら、XMLファイルとセットなのかも
			dataFile = query_info->GetQueryPair("XMLFile");
			curData["FolderKey"]	= folderKey;
			curData["FileName"]		= dataFile;
			curData["DataKey"]		= "\'" + folderKey + "\', \'" + dataFile + "\', \'" + curData["KeyValue"] + "\'";

			textData = new TextDataFile();
			// データ読み込みを行う
			dataFullPath = dataFolder/* + dataFile*/;
		}
		else{
/*
			ConvertGrammar	*grammar = new ConvertGrammar();
			textData = grammar;
*/
			textData = new ConvertGrammar();

			dataFullPath = query_info->GetQueryPair("XMLFile");
			dataFile = dataFullPath;

			curData["DataKey"]		= "\'" + folderKey + "\', \'" + dataFile + "\', \'" + curData["KeyValue"] + "\'";
		}

		{
			std::string		buff;
			field_value		recordData;
			record_it		recData;

			textData->openFile(dataFullPath);
			if(textData->readData()){
				recordData.clear();
				recordData = textData->getFields();

				// フィールド名退避
				buff = "<tr>";

				// レコード削除の際の削除レコードチェック欄を追加
				if(query_info->GetQueryPair("RecDel") == "OK"){
					buff = buff + "<th>" + "</th>";
				}

				for(str_it it = recordData.begin(); it != recordData.end(); it++){
//					std::cout << *it << std::endl;
					buff = buff + "<th>" + (std::string)(*it) + "</th>";
				}

				// 編集画面へのリンクを設定する？
				if(query_info->GetQueryPair("RecMod") != "NG"){
					buff = buff + "<th></th>";
				}

				buff = buff + "</tr>\n";
				curData["FieldHeaders"] = buff;
/*
				recData = textData->getTopRecord();
				for(; recData != (record_it)NULL; ){
					for(str_it it = recordData.begin(); it != recordData.end(); it++){
						// レコード情報退避？？
						std::cout << (recData->second)[*it] << std::endl;
					}
					recData = textData->getNextRecord();
				}
*/
			}
			else
				std::cout << "データしっぱい<br />" << std::endl;

		}
	}

	{
		std::string		params;

		params ="\'" + folderKey + "\', " + "\'" + dataFile + "\'";
		curData["FunctionBar"] = "";

		if(query_info->GetQueryPair("RecAdd") == "OK")
			curData["FunctionBar"] = curData["FunctionBar"] + "<input type = \"button\" value = \"新規\" onClick=\"AddData(" + params + ")\"  > \n";
		else
			curData["FunctionBar"] = curData["FunctionBar"] + "<input type = \"button\" disabled value = \"新規\" onClick=\"AddData()\"  > \n";

		if(query_info->GetQueryPair("RecDel") == "OK")
			curData["FunctionBar"] = curData["FunctionBar"] + "<input type = \"button\" value = \"削除\" onClick=\"DelData(" + params + ")\"  > \n";
		else
			curData["FunctionBar"] = curData["FunctionBar"] + "<input type = \"button\" disabled value = \"削除\" onClick=\"DelData()\"  > \n";

		curData["FunctionBar"] = curData["FunctionBar"] + "<input type = \"button\" value = \"検索\" onClick=\"FindData(" + params + ")\" > \n";
	}

	// データクエリを設定する
	if(query_info->GetQueryPair("WhereStr").length() > 0){
		textData->setQuery(query_info->GetQueryPair("WhereStr"));
	}

	return true;
}

std::string SystemControl::getMapData(std::string keyName)
{
	return curData[keyName];
}

std::string	SystemControl::getHTMLFileName(std::string pageName, std::string command)
{
#ifdef _DEBUG
fprintf(pDbgLog, "Mod HTMLFileName = %s\n", pageName.c_str());
fflush(pDbgLog);
#endif
#ifdef GRAMMAR_COMBINATION
	ConvertGrammar	*xmlData = 0;
//std::cout << "出力？ " << dataFile << "<br />" << std::endl;
	if((curData["FolderKey"].length() > 0) && (dataFile.length() > 0)){
		// テキストデータファイルがターゲットになっていて、XMLファイルもターゲットになっている
		xmlData = new ConvertGrammar();
		xmlData->openFile(dataFile);
		xmlData->readData();
	}
#endif
	if(pageName.compare("UpdateData") == 0){
		field_value		recordData, fieldData;
		record_it		recData;
		field_data		dataRecord;

		// データ更新を行い、結果を[UpdateResult]に格納する。
		fieldData = textData->getFields();

//		dataRecord = (*textData)[query_info->GetQueryPair("KeyValue")];

		for(str_it it = fieldData.begin(); it != fieldData.end(); it++){
			recordData.push_back(query_info->GetQueryPair((std::string)(*it)));
		}

		textData->writeRecord(query_info->GetQueryPair("KeyValue"), recordData);
		textData->writeData();

#ifdef GRAMMAR_COMBINATION
		if(xmlData != 0){
			recordData.clear();
			fieldData = xmlData->getFields();
			for(str_it it = fieldData.begin(); it != fieldData.end(); it++){
				recordData.push_back(query_info->GetQueryPair((std::string)(*it)));
			}
			xmlData->writeRecord("", recordData);
			xmlData->writeData();
//std::cout << "出力した" << "<br />" << std::endl;
		}
#endif

	}
	if(pageName.compare("DelData") == 0){
		// DBからレコードを削除する
		std::string	delKey = query_info->GetQueryPair("KeyValue");
		std::vector< std::string > delKeyList = string_token(delKey, " ,");

		for(std::vector< std::string >::iterator it = delKeyList.begin(); it != delKeyList.end(); it++){
			// 削除対象データを取得して、XML用のキーを取り出す
#ifdef GRAMMAR_COMBINATION
			if(xmlData != 0){
				xmlData->removeRecord((*textData)[*it]["grapheme"]);
			}
#endif
			// 実際にレコード削除を実行する
			textData->removeRecord(*it);
		}
		textData->writeData();
#ifdef GRAMMAR_COMBINATION
		if(xmlData != 0){
			xmlData->writeData();
		}
#endif
	}

#ifdef GRAMMAR_COMBINATION
	if(xmlData != 0)
		delete xmlData;
#endif

	return htmlList[pageName];
}

bool SystemControl::loopProc(std::string loopKey)
{
	std::string		buff;
	field_value		recordData;
	record_it		recData;
	field_data		dataRecord;
	std::string		mainKey = "";

	if(loopKey.compare("records") == 0){
		bool			dateFlg = false;
		std::string		dateField = "", timeField = "";
		std::vector< SortData >	sortList;

		recordData = textData->getFields();
		dateField = recordData[0];
		mainKey = dateField;

		for(str_it it = recordData.begin(); it != recordData.end(); it++){
			if(textData->isDate(*it)){
				dateFlg = true;
				dateField = *it;
			}
			if(textData->isTime(*it)){
				timeField = *it;
			}
			if((dateField.length() > 0) && (timeField.length() > 0))
				break;
		}

		recData = textData->getTopRecord();
		// ソート用の配列を作る
		for(; recData != (record_it)NULL; ){
			SortData	sortData;

			dataRecord = recData->second;
			sortData.key = dataRecord[mainKey];
			if(timeField.length() > 1)
				sortData.sort_key = DatetimeStrToDouble(dataRecord[dateField], dataRecord[timeField]);
			else
				sortData.sort_key = DatetimeStrToDouble(dataRecord[dateField], "");

			sortList.push_back(sortData);
			recData = textData->getNextRecord();
		}
		// ソートを実行する
		if(dateFlg)
			sort(sortList.begin(), sortList.end(), std::greater<SortData>());
		else
			sort(sortList.begin(), sortList.end());

		for(std::vector< SortData >::iterator sort_it = sortList.begin(); sort_it != sortList.end(); sort_it++){
			mainKey = "";
			buff = "";

			dataRecord = (*textData)[sort_it->key];
			for(str_it it = recordData.begin(); it != recordData.end(); it++){
				// レコード情報退避？？
				buff = buff + "<td>" + (std::string)(dataRecord[*it]) + "</td>\n";
				if(mainKey.length() == 0)	mainKey = dataRecord[*it];
			}

			// レコード削除の際の削除レコードチェック欄を追加
			if(query_info->GetQueryPair("RecDel") == "OK"){
				buff = "<td><input type = \'checkbox\' name=\'" + mainKey+ "\' id=\'" + mainKey + "\'></td>" + buff;
			}

			// 編集用リンクを作る
			if(query_info->GetQueryPair("RecMod") != "NG"){
				buff = buff + "<td><input type=\"button\" value=\"編集\" onClick=\"ModData(";
				buff = buff + "\'" + folderKey + "\', " + "\'" + dataFile + "\', " + "\'" + mainKey + "\')\"></td>";
			}

			buff = "<tr>" + buff + "</tr>";
			curData["FieldValues"] = buff;
			writeLine();
		}
		return true;
	}

	if(loopKey.compare("mod_data") == 0){
		recordData = textData->getFields();
		dataRecord = (*textData)[query_info->GetQueryPair("KeyValue")];
		for(str_it it = recordData.begin(); it != recordData.end(); it++){
			curData["FieldHeaders"]	= *it;
			if(dataRecord.find(*it) != dataRecord.end()){
				curData["FieldValues"]	= dataRecord[*it];
				curData["Attribute"]	= textData->isModify(*it) == true ? "" : "disabled";
			}
			else{
				curData["FieldValues"]	= "";
				if(textData->isDate(*it))
					curData["FieldValues"]	= GetDateStr();
				if(textData->isTime(*it))
					curData["FieldValues"]	= GetTimeStr();
				curData["Attribute"]	= textData->isAdd(*it) == true ? "" : "disabled";
			}
			writeLine();
		}

		return true;
	}

	if(loopKey.compare("filter_data") == 0){
		recordData = textData->getFields();
//		dataRecord = (*textData)[query_info->GetQueryPair("KeyValue")];
		for(str_it it = recordData.begin(); it != recordData.end(); it++){
			curData["FieldHeaders"]	= *it;
//			if(dataRecord.find(*it) != dataRecord.end())
//				curData["FieldValues"]	= dataRecord[*it];
//			else
				curData["FieldValues"]	= "";
			writeLine();
		}

		return true;
	}

	return false;
}
