#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Jabber(XMPP) messaging component
#           copyright 2010 Yosuke Matsusaka, AIST.

import sys
import locale
import codecs
import traceback

import OpenRTM_aist
import RTC

from pyxmpp.all import JID, Message
from pyxmpp.jabber.client import JabberClient
from pyxmpp.streamtls import TLSSettings

from xml.dom import minidom
from xml.dom.minidom import Document
from BeautifulSoup import BeautifulSoup

class Client(JabberClient):
    def __init__(self, jid, password, obj):
        if not jid.resource:
            jid = JID(jid.node, jid.domain, "JabberRTC")
        tls = TLSSettings(require = True, verify_peer = False)
        auth = ['sasl:PLAIN']
        JabberClient.__init__(self, jid, password, tls_settings = tls, auth_methods = auth)
        self._rtobj = obj

    def session_started(self):
        JabberClient.session_started(self)
        self.stream.set_presence_handler(None, self.presence)
        self.stream.set_presence_handler("unavailable", self.presence)
        self.stream.set_presence_handler("subscribe", self.presence_control)
        self.stream.set_presence_handler("subscribed", self.presence_control)
        self.stream.set_presence_handler("unsubscribe", self.presence_control)
        self.stream.set_presence_handler("unsubscribed", self.presence_control)
        self.stream.set_message_handler("normal",self.message)

    def message(self, stanza):
        f = stanza.get_from().as_unicode()
        body = stanza.get_body()
        if body is not None:
            print u'%s has send you message: %s' % (f, body)
            doc = Document()
            msg = doc.createElement("message")
            msg.setAttribute("from", f)
            msg.setAttribute("body", body)
            doc.appendChild(msg)
            self._rtobj._outdata.data = doc.toxml(encoding="utf-8")
            self._rtobj._outport.write()

    def sendmessage(self, to, body):
        print u'send message to %s: %s' % (to, body)
        self.stream.send(Message(to_jid = JID(to), body = body))

    def presence(self, stanza):
        f = stanza.get_from().as_unicode()
        state = "available"
        if stanza.get_type() == "unavailable":
            state = "unavailable"
        if stanza.get_show() == "away":
            state = "away"
        print u"%s has become %s" % (f, state)
        doc = Document()
        msg = doc.createElement("status")
        msg.setAttribute("from", f)
        msg.setAttribute("state", state)
        doc.appendChild(msg)
        self._rtobj._statedata.data = doc.toxml(encoding="utf-8")
        self._rtobj._stateport.write()

    def presence_control(self,stanza):
        return stanza.make_accept_response()


pyJabber_spec = ["implementation_id", "pyJabber",
                 "type_name",         "pyJabber",
                 "description",       "Jabber(XMPP) messaging component (python implementation)",
                 "version",           "1.0.0",
                 "vendor",            "AIST",
                 "category",          "communication",
                 "activity_type",     "DataFlowComponent",
                 "max_instance",      "10",
                 "language",          "Python",
                 "lang_type",         "script",
                 "conf.default.id",   "[your id]@gmail.com",
                 "conf.default.password", "[your password]"
                 ""]

class DataListener(OpenRTM_aist.ConnectorDataListenerT):
    def __init__(self, name, obj):
        self._name = name
        self._obj = obj
    
    def __call__(self, info, cdrdata):
        data = OpenRTM_aist.ConnectorDataListenerT.__call__(self, info, cdrdata, RTC.TimedString(RTC.Time(0,0),""))
        self._obj.onData(self._name, data)

class pyJabber(OpenRTM_aist.DataFlowComponentBase):
    def __init__(self, manager):
        try:
            OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)
            self._c = None
            self._jid = ["",]
            self._password = ["",]
        except:
            print traceback.format_exc()

    def onInitialize(self):
        try:
            # bind configuration parameters
            self.bindParameter("id", self._jid, "[your id]@gmail.com")
            self.bindParameter("password", self._password, "[your password]")
            # create inport
            self._indata = RTC.TimedString(RTC.Time(0,0), "")
            self._inport = OpenRTM_aist.InPort("text", self._indata)
            self._inport.addConnectorDataListener(OpenRTM_aist.ConnectorDataListenerType.ON_BUFFER_WRITE,
                                                  DataListener("ON_BUFFER_WRITE", self))
            self.registerInPort("text", self._inport)
            # create outport for message
            self._outdata = RTC.TimedString(RTC.Time(0,0), "")
            self._outport = OpenRTM_aist.OutPort("message", self._outdata)
            self.registerOutPort("message", self._outport)
            # create outport for status
            self._statedata = RTC.TimedString(RTC.Time(0,0), "")
            self._stateport = OpenRTM_aist.OutPort("status", self._statedata)
            self.registerOutPort("status", self._stateport)
        except:
            print traceback.format_exc()
        return RTC.RTC_OK

    def onActivated(self, ec_id):
        try:
            self._c = Client(JID(self._jid[0]), self._password[0], self)
            self._c.connect()
        except:
            print traceback.format_exc()
        return RTC.RTC_OK
    
    def onDeactivate(self, ec_id):
        try:
            if self._c:
                self._c.disconnect()
                self._c = None
        except:
            print traceback.format_exc()
        return RTC.RTC_OK

    def onData(self, name, data):
        try:
            doc = BeautifulSoup(data.data)
            if doc:
                to = doc.first()["to"]
                body = doc.first()["body"]
                self._c.sendmessage(to, body)
        except:
            print traceback.format_exc()

    def onResult(self, data):
        pass
    
    def onExecute(self, ec_id):
        try:
            if self._c:
                self._c.loop(1)
        except:
            print traceback.format_exc()
        return RTC.RTC_OK

    def onFinalize(self):
        if self._c:
            self._c.disconnect()
            self._c = None
        return RTC.RTC_OK

class pyJabberRTCManager:
    def __init__(self):
        self._comp = None
        self._manager = OpenRTM_aist.Manager.init(sys.argv)
        self._manager.setModuleInitProc(self.moduleInit)
        self._manager.activateManager()

    def start(self):
        self._manager.runManager(False)

    def moduleInit(self, manager):
        profile=OpenRTM_aist.Properties(defaults_str=pyJabber_spec)
        manager.registerFactory(profile, pyJabber, OpenRTM_aist.Delete)
        self._comp = manager.createComponent("pyJabber?exec_cxt.periodic.rate=0.1")

def main():
    locale.setlocale(locale.LC_CTYPE, "")
    encoding = locale.getlocale()[1]
    if not encoding:
        encoding = "us-ascii"
    sys.stdout = codecs.getwriter(encoding)(sys.stdout, errors = "replace")
    sys.stderr = codecs.getwriter(encoding)(sys.stderr, errors = "replace")
    manager = pyJabberRTCManager()
    manager.start()

if __name__=='__main__':
    main()
