/*
 * SystemStatus.cpp
 *
 *  Created on: 2011/01/13
 *      Author: yotubata
 */
#include "SystemStatus.h"
#include "iostream"
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "Tools.h"
#include "TSProperty.h"
#include "TextData.h"


static const long id_mask[] = {
	0x00000000,
	0xFFFF00FF,
	0xFF000000,
	0x00FF0000,
	0x0000FF00,
	0x000000FF,
	0x00FFFF00,
	0xFFFF0000,
	0x0000FFFF,
	0xFF0000FF,
	0xFF00FF00,
	0x00FF00FF,
	0xFFFFFF00,
	0x00FFFFFF,
	0xFF00FFFF,
	0xFFFFFFFF
};

static const char* system_status_string[] = {
	"01	待機状態",
	"02	起動中",
	"03	来訪者検出通知",
	"04	受付中",
	"05	連絡中",
	"06	給仕操作開始(ユーザー操作中)",
	"07	給仕中(ロボット動作中)",
	"08	廃棄操作開始(ユーザー操作中)",
	"09	廃棄中(ロボット動作中)",
	"10	サービス実行中",
	"11	移動中",
	"12	積載待機中",
	"13	回収待機中",
	"14	給仕中",
	"15	廃棄中",
	"99	システムエラー",
	""
};

static const char* system_error_string[] = {
	"00	正常稼働",
	"01	コンポーネント未起動",
	"02	受付端末動作異常",
	"03	会議室端末動作異常",
	"04	音声コンポーネント異常",
	"05	GTalkクライアント異常",
	"06	ポーリング異常",
	"07	RH位置不正",
	"08	ドリンク在庫無し",
	"09	サービス動作中",
	"10	非常停止",
	"11	走行不能",
	"12	バッテリー残量低下",
	"13	給仕サービス動作中",
	"14	廃棄サービス動作中",
	"91	コンポーネントエラー"
	""
};

static const char* system_state_map[]{
	"0x01FF0001, 1",
	"0x01FF0002, 14",
	"0x01FF0003, 7",
	"0x01FF0004, 15",
	"0x01FF0005, 9",
	"0x01FF0006, 12",
	"0x01FF0007, 6",
	"0x01FF0008, 8",
	"0x02FF0000, 1",
	"0x02FF0001, 1",
	"0x02FF0002, 2",
	"0x02FF0003, 3",
	"0x02FF0004, 4",
	"0x02FF0005, 5",
	"0x03FF0000, 1",
	"0x03FF0001, 1",
	"0x03FF0002, 6",
	"0x03FF0003, 7",
	"0x03FF0004, 8",
	"0x03FF0005, 9",
	"0x04FF0000, 1",
	"0x04FF0001, 1",
	"0x04FF0002, 2",
	"0x04FF7002, 10",
	"0x04FFFF02, 10",
	"0x04FF0003, 10",
	"0x05FF0000, 1",
	"0x05FF0001, 1",
	"0x05FF0002, 2",
	"0x05FF0003, 11",
	"0x05FF0004, 12",
	"0x05FF0005, 13",
	"0x05FF0006, 14",
	"0x05FF0007, 15",
	""
};

static const char* system_error_map[]{
	/* エラー番号 ID 通知先(1:担当, 2:受付, 4:会議室, FF:全部) 制御HTML 受付HTML 会議室HTML */
	"0xFFFF00FF, 00, 1, ",

	"0x01FFFF00, 00, 0, ",
	"0x01FF0001, 00, 0, ",
	"0x01FFFF01, 01, 0, ",
	"0x01FFFF02, 02, 0, ",
	"0x01FFFF03, 03, 0, ",
	"0x01FFFF04, 04, 0, ",
	"0x01FFFF05, 05, 0, ",
	"0x01FFFFFF, 91, 1, ",

	"0x02FFFF00, 00, 0, ",
	"0x02FF0001, 00, 0, ",
	"0x02FFFF01, 01, 0, ",
	"0x02FFFF02, 06, 0, ",
	"0x02FFFFFF, 91, 1, ",

	"0x03FFFF00, 00, 0, ",
	"0x03FF0001, 00, 0, ",
	"0x03FFFF01, 06, 0, ",
	"0x03FFFFFF, 91, 1, ",

	"0x04FFFF00, 00, 0, ",
	"0x04FF0001, 00, 0, ",
	"0x04FFFF01, 01, 5, ",
	"0x04FFFF02, 07, 1, ",
	"0x04FFFF03, 08, 5, Drink",
	"0x04FFFF04, 09, 0, ",
	"0x04FFFF05, 10, 5, ",
	"0x04FFFFFF, 91, 1, ",

	"0x05FFFF00, 00, 0, ",
	"0x05FF0001, 00, 5, ",
	"0x05FFFF01, 01, 5, ",
	"0x05FFFF02, 11, 5, DriveImpossible",
	"0x05FFFF03, 12, 5, Battery",
	"0x05FFFF04, 13, 0, ",
	"0x05FFFF05, 14, 0, ",
	"0x05FFFF06, 10, 5, ",
	"0x05FFFFFF, 91, 1, ",

	"0xFFFFF0FF, 91, 1, ",
	""
};


SystemDataMgr::SystemDataMgr()
{
	shmpt			= 0;
	systemStatus	= 0;
	errorTerminal	= 0;
}

SystemDataMgr::~SystemDataMgr()
{
	destroyData();
}

CurrentStatusInfo* SystemDataMgr::createData()
{
	int					shmid;

	if(shmpt == 0){
		// 共有メモリの作成を行う
		shmid = shmget(DATA_CONT_ID, sizeof(CurrentStatusInfo), IPC_CREAT | IPC_EXCL | 0666);
		if(shmid < 0){
			if(errno == EEXIST){
				// 既存共有メモリを取得する
				shmid = shmget(DATA_CONT_ID, sizeof(CurrentStatusInfo), 0666);
			}
			if(shmid < 0){
				std::cout << "共有メモリ初期化エラー" << std::endl;
				printf( "エラー(%d):%s\n", errno, strerror(errno));
				return 0;
			}
		}

		shmpt = (CurrentStatusInfo *)shmat(shmid, 0, 0);
		if(shmpt == 0){
			std::cout << "共有メモリ確保エラー" << std::endl;
			return 0;
		}
		memset(shmpt, 0, sizeof(CurrentStatusInfo));
	}
	else{

	}
/*
	status_str			statusStrList;
	status_str			errorStrList;
	status_value		statusList;
	error_value			errorList;
*/
	if( statusStrList.size() == 0){
		readStrList(system_status_string, &statusStrList);
	}

	if( errorStrList.size() == 0){
		readStrList(system_error_string, &errorStrList);
	}

	if( statusList.size() == 0){
		readStatusList(system_state_map);
	}

	if( errorList.size() == 0){
		readErrorList(system_error_map);
	}


	return shmpt;
}

void SystemDataMgr::readStrList(const char** list, status_str* map_buff)
{
	std::vector< std::string > token;

	for(int n = 0; strlen(list[n]) != 0; n++){
		// 文字列をトークンに分解
		token = string_token(list[n], " \t,");

		// マップ情報生成
		if(token.size() == 2){
			map_buff->insert(status_pair(string_to_int(token[0]), token[1]));
		}
	}
}
void SystemDataMgr::readStatusList(const char** list)
{
	std::vector< std::string > token;

	for(int n = 0; strlen(list[n]) != 0; n++){
		// 文字列をトークンに分解
		token = string_token(list[n], " \t,");

		// マップ情報生成
		if(token.size() == 2){
			statusList.insert(value_pair(string_to_int(token[0]), string_to_int(token[1])));
		}
	}
}
void SystemDataMgr::readErrorList(const char** list)
{
	std::vector< std::string > token;

	for(int n = 0; strlen(list[n]) != 0; n++){
		// 文字列をトークンに分解
		token = string_token(list[n], " \t,");

		// マップ情報生成
		if(token.size() > 2){
			ErrorInfo	errInfo;

			errInfo.errStrID	= string_to_int(token[1]);
			errInfo.notifyInfo	= string_to_int(token[2]);
			errInfo.errRecoveryURL.resize(3, "");
			for(int recoveryCnt = 3; token.size() > recoveryCnt; recoveryCnt++){
				errInfo.errRecoveryURL[0] = (token[recoveryCnt]);
			}
//			printf("%x\n", string_to_int(token[0]));
			errorList.insert(error_pair(string_to_int(token[0]), errInfo));
		}
	}
}


bool SystemDataMgr::clearData()
{
	memset(shmpt->scenarioID, 0, sizeof(shmpt->scenarioID));
	memset(shmpt->scenarioFileName, 0, sizeof(shmpt->scenarioFileName));

	return true;
}

bool SystemDataMgr::destroyData()
{
	int		sysRet;

	// 共有メモリの破棄を行う
	// 参照カウンタが0になったら削除
	sysRet = shmdt(shmpt);
	shmpt = 0;

	return true;
}

bool SystemDataMgr::isNewScenario()
{
	return (strlen(shmpt->scenarioID) == 0);
}

bool SystemDataMgr::getStatus(CurrentStatusInfo *info)
{
	if(shmpt != 0){
		*info = *shmpt;
		return true;
	}

	return false;
}

bool SystemDataMgr::setStatus(CurrentStatusInfo state)
{
	if(shmpt != 0){
		*shmpt = state;
		return true;
	}

	return false;
}

bool SystemDataMgr::setScenarioID(std::string scenarioID)
{
	if(shmpt != 0){
		strncpy(shmpt->scenarioID, scenarioID.c_str(), sizeof(shmpt->scenarioID));
		return true;
	}

	return false;
}

bool SystemDataMgr::setScenarioFileName(std::string scenarioFileName)
{
	if(shmpt != 0){
		strncpy(shmpt->scenarioFileName, scenarioFileName.c_str(), sizeof(shmpt->scenarioFileName));
		return true;
	}

	return false;
}

bool SystemDataMgr::setScenarioPosition(unsigned long index)
{
	if(shmpt != 0){
		shmpt->runPosition = index;
		return true;
	}

	return false;
}

bool SystemDataMgr::setSystemStatus(SystemStatus status)
{
	if(shmpt != 0){
		shmpt->systemStatus = status;
		return true;
	}

	return false;
}

std::string SystemDataMgr::getErrorString(long id)
{
	std::string	recoveryPage	= "";
	std::string result			= "不明";
//printf("ErrorID = %08x\t", id);

	for(long idCnt = 0; id_mask[idCnt] != 0xFFFFFFFF; idCnt++){
//		printf("%x / %x = ", id, id_mask[idCnt]);
		if(errorList.find(id | id_mask[idCnt]) != errorList.end()){
			// 定義されている
//			printf("%x\n", id | id_mask[idCnt]);
			result = errorStrList[errorList[(id | id_mask[idCnt])].errStrID];
			recoveryPage = errorList[(id | id_mask[idCnt])].errRecoveryURL[0];
			break;
		}
//		printf("\n");
	}

	if(result.length() == 0)	result = "未定義";

	if((id & 0x0000FF00) > 0){
		systemStatus	= id;
		errorTerminal	= 0xFFFF0000 & id;
		recoveryURL		= recoveryPage;
	}
	else{
		if((0xFFFF0000 & id) == errorTerminal){
			id = 0;
			recoveryURL = "";
		}
	}

	return result;
}

std::string SystemDataMgr::getStatusString(long val)
{
	std::string result = "不明";

//printf("StateID = %08x\t", val);

	for(int idCnt = 0; id_mask[idCnt] != 0xFFFFFFFF; idCnt++){
		if(statusList.find(val | id_mask[idCnt]) != statusList.end()){
			// 代替定義されている
			result = statusStrList[statusList[val | id_mask[idCnt]]];
			break;
		}
	}

	if(result.length() == 0)	result = "未定義";

	return result;
}

bool SystemDataMgr::WriteLogData(std::vector< long > stateData)
{
	field_value		recordData, fieldData;
	TS_Property		systemProperty;
	TextDataFile	textData;
	std::string		dataFolder;

	dataFolder = systemProperty["SystemLog"];
	if(dataFolder.length() == 0) dataFolder = "/tmp/SystemLog.log";

	textData.openFile(dataFolder);
	textData.readData();

	// データ更新を行い、結果を[UpdateResult]に格納する。
	fieldData = textData.getFields();

	recordData.push_back("");

	// 日付
	recordData.push_back(GetDateStr());
	// 時刻
	recordData.push_back(GetTimeStr());

	// サービス
	recordData.push_back(getStatusString(stateData[0]));
	// 制御端末
	recordData.push_back(getErrorString(stateData[1]));
	// 受付端末
	recordData.push_back(getErrorString(stateData[2]));
	// 会議室端末
	recordData.push_back(getErrorString(stateData[3]));
	// RH
	recordData.push_back(getErrorString(stateData[4]));
	// PA10
	recordData.push_back(getErrorString(stateData[5]));

	textData.writeRecord("", recordData);

	textData.writeData();

	return true;
}

/*
bool SystemDataMgr::setTerminalType(TerminalType type)
{
	if(shmpt != 0){
		shmpt->terminalType = type;
		return true;
	}

	return false;
}
*/
