/*
 * TextData.cpp
 *
 *  Created on: 2010/12/10
 *      Author: yotubata
 */
#include <iostream>
#include <unistd.h>

#include "TextData.h"
#include "Tools.h"

TextDataFile::TextDataFile(const std::string fileName, const bool writeMode, const unsigned long maxRecord)
{
	if(writeMode){
		if(fileName.length() > 0){
			field_value	fields;
			createFile(fileName, fields);
		}
	}
	else{
		if(fileName.length() > 0){
			openFile(fileName);
		}
	}

	buffSize = maxRecord;
}

bool TextDataFile::openFile(const std::string fileName)
{
	bool	result = true;

	// オープン済みか確認する
	if(ifsDataFile.is_open() || ofsDataFile.is_open()){
		// オープン済みなので、ファイル名を比較する
		// 同一名でも開き直す
		/*if(dataFileName != fileName)*/{
			// ファイル名が違うので、以前のは破棄して作り直す
			if(ifsDataFile.is_open())	ifsDataFile.close();
			if(ofsDataFile.is_open())	ofsDataFile.close();
			if(fileName.length() > 0)
				dataFileName = fileName;
			openFile(dataFileName);
		}
	}
	else{
		// まだオープンしてない
		// ファイルの存在を確認し、存在している場合は読み込みで開く
		if(access(fileName.c_str(), F_OK) == 0){
			// ファイルストリームを作成する
			ifsDataFile.open(fileName.c_str(), std::ios::in);
			if(ifsDataFile){
				dataFileName = fileName;
				fieldNameList.clear();
				recordBuff.clear();
			}
			else{
				std::cout << "Read file open error!" << std::endl;
				result = false;
			}
		}
	}
	return result;
}

/*
 * @brief データファイルを作成する
 */
bool TextDataFile::createFile(const std::string fileName, const field_value dataFieldName)
{
	bool	result = false;

	// 既にファイルがないか確認する
#if 0
	if(access(fileName.c_str(), F_OK) == 0){
		// あるようなので、エラーにする
		std::cout << "File already exist!" << std::endl;
		unlink(fileName.c_str());
	}
#endif

	// ファイルストリームを作成する
	ofsDataFile.open(fileName.c_str(), std::ios::out);
	if(ofsDataFile){
		dataFileName = fileName;

//		fieldNameList = dataFieldName;
		copyFieldData(dataFieldName, fieldNameList);
	}
	else{
		std::cout << "Write file open error!" << std::endl;
		result = false;
	}

	return result;
}


bool TextDataFile::createFile(const std::string fileName, const field_list dataFieldName)
{
	bool	result = false;
#if 0
	// 既にファイルがないか確認する
	if(access(fileName.c_str(), F_OK) == 0){
		// あるようなので、エラーにする
		std::cout << "File already exist!" << std::endl;
		unlink(fileName.c_str());
	}
#endif
	// ファイルストリームを作成する
	ofsDataFile.open(fileName.c_str(), std::ios::out);
	if(ofsDataFile){
		dataFileName = fileName;

		fieldNameList = dataFieldName;
	}
	else{
		std::cout << "Write file open error!" << std::endl;
		result = false;
	}

	return result;
}

void TextDataFile::copyFieldData(/*const */field_value fieldVal, field_list fieldList)
{
	FieldData	fieldData;

	fieldList.clear();
	for(str_it it = fieldVal.begin(); it != fieldVal.end(); it++){
		fieldData.setFieldName(*it);
	}
	return;
}


bool TextDataFile::writeFields()
{
	std::string	recordString = "";

	// フィールド名を出力する
	for(field_list_it it = fieldNameList.begin(); it != fieldNameList.end(); it++){
		recordString = recordString + (*it).fieldName_org;
		ofsDataFile << recordString;
		recordString = "\t";
	}
	ofsDataFile << std::endl;

	return true;
}

bool TextDataFile::writeRecord()
{
	std::string		recordString = "";
	field_data		r_data;

	for(record_it it = recordBuff.begin(); it != recordBuff.end(); it++){
		r_data = it->second;
		recordString = "";
		// １レコード処理
		for(field_list_it fit = fieldNameList.begin(); fit!= fieldNameList.end(); fit++){
			// フィールド分回る
			recordString = recordString + (std::string)(r_data[(*fit).fieldName]);
			ofsDataFile << recordString;
			recordString = "\t";
		}
		ofsDataFile << std::endl;
	}

	return true;
}

int TextDataFile::readData()
{
	bool		result = true;
	std::string	recordString;
	field_value	recordData;
	int	index;

	if(!(ifsDataFile.is_open()))	return false;

	// ファイルを読み込んで、フィールド一覧とレコードを作成する
	// 先頭行はフィールド名の列挙
	// 2行目以降がデータとなるが、先頭が「#;」のどれかの文字で始まる場合はコメント行とする
	// ファイルを読み込んで、シナリオリストを生成する
	curRecord = (record_it)NULL;
	while(getline(ifsDataFile, recordString)){
		if(recordString.length() > 1){
			// コメント行は削除する
			index = recordString.find(";");
			if(index == 0){
				continue;
			}
			index = recordString.find("#");
			if(index == 0){
				continue;
			}
			// データあり
			if(recordString.length() > 1){
				recordData = string_token(recordString, "\t");
				// フィールド一覧作成
				if(fieldNameList.size() > 0){
					int		fieldCnt = 0;
					field_data	oneRecord;
					// データ追加
					for(str_it it = recordData.begin(); it != recordData.end(); it++){
						// データ収納
						oneRecord.insert(field_pair(fieldNameList[fieldCnt].fieldName, *it));
						fieldCnt++;
					}
					recordBuff.insert(record_pair(recordData[0], oneRecord));
				}
				else{
					// フィールド名取得
					for(str_it it = recordData.begin(); it != recordData.end(); it++){
						// データ収納
						if(it->length() > 0) {
							FieldData	fieldInfo;

							fieldInfo.setFieldName(*it);
							fieldNameList.push_back(fieldInfo);
						}
					}
				}
			}
		}
	}

/*
	// ダンプ
	std::cout << "ReadData!" << std::endl;
	for(record_it rit = recordBuff.begin(); rit != recordBuff.end(); rit++){
		for(field_it fit = rit->second.begin(); fit != rit->second.end(); fit++){
			std::cout << fit->second;
		}
		std::cout << std::endl;
	}
	std::cout << "ReadData End!" << std::endl;
*/

	return result;
}

bool TextDataFile::isModify(std::string	fieldName)
{
	return FindFieldData(fieldName).modFlg;
}

bool TextDataFile::isAdd(std::string	fieldName)
{
	return FindFieldData(fieldName).addFlg;
}

bool TextDataFile::isRange(std::string	fieldName)
{
	return FindFieldData(fieldName).rangeFilter;
}

bool TextDataFile::isDate(std::string	fieldName)
{
	return (FindFieldData(fieldName).fieldType == ftDate);
}
bool TextDataFile::isTime(std::string	fieldName)
{
	return (FindFieldData(fieldName).fieldType == ftTime);
}

FieldData TextDataFile::FindFieldData(std::string fieldName)
{
	FieldData	result;

	for(field_list_it fit = fieldNameList.begin(); fit!= fieldNameList.end(); fit++){
		// フィールド分回る
		if( (*fit).fieldName == fieldName ){
			result = *fit;
			break;
		}
	}

	return result;
}

bool TextDataFile::addField(std::string fieldName)
{
	FieldData	fieldData;

	fieldData.fieldName_org = fieldName;
	fieldNameList.push_back(fieldData);

	return true;
}

int TextDataFile::writeData(const std::string fileName)
{
	std::string	writeFileName = dataFileName;

	if(fileName.length() > 0)
		writeFileName = fileName;

	if(ifsDataFile.is_open())	ifsDataFile.close();

	if(ofsDataFile.is_open()){
		// オープン済みなので、ファイル名を比較する
		if(dataFileName != writeFileName){
			// ファイル名が違うので、以前のは破棄して作り直す
			if(ifsDataFile.is_open())	ifsDataFile.close();
			if(ofsDataFile.is_open())	ofsDataFile.close();
			dataFileName = writeFileName;
			createFile(writeFileName, fieldNameList);
		}
	}
	else{
		createFile(writeFileName, fieldNameList);
	}
	if(fieldNameList.size() > 0){
		writeFields();
		writeRecord();
	}

	return 0;
}

bool TextDataFile::writeRecord(const std::string key, const field_value recordData)
{
	field_value		recordWork = recordData;
	std::string		keyVal = key;
	bool			result = true;
	int				fCnt;

	fCnt = 0;
//	if(recordBuff.find(key) != 0){
	if(key.length() == 0){
#if 0
		// キーが指定されなかった場合は、IDの最大値を使用して格納する??
		// std::mapの特性なのかは判らんが、キーの昇順に格納されているようなので、その並び順を利用する。
		int		curNum = 1;

		for(record_it it = recordBuff.begin(); it != recordBuff.end(); it++){
			if(string_to_int(it->first) != curNum){
				keyVal = int_to_string(curNum);
				recordWork[0] = keyVal;
				break;
			}
			curNum++;
		}
#else
		for(size_t curNum = 1 ; curNum < (recordBuff.size() + 2) ; curNum++){
			keyVal = int_to_string(curNum);
			if(recordBuff.find(keyVal) == recordBuff.end()){
				recordWork[0] = keyVal;
				break;
			}
		}
#endif
	}

	for(field_list_it fit = fieldNameList.begin(); fit!= fieldNameList.end(); fit++){
		// フィールド分回る
//		std::cout << keyVal << " / " << (*fit).fieldName << " / " << fCnt << " / " << recordWork[fCnt] << std::endl;
		recordBuff[keyVal][(*fit).fieldName] = recordWork[fCnt];
		fCnt++;
	}

	return result;
}

field_data& TextDataFile::operator[](std::string key)
{
	dummy_data.clear();

	// 該当データ在りか確認する
	if(recordBuff.find(key) != recordBuff.end())
		return recordBuff[key];
	else
		return dummy_data;
}

bool TextDataFile::removeRecord(const std::string key)
{
	int		delCnt;

	delCnt = recordBuff.erase(key);

	return (delCnt > 0);
}

bool TextDataFile::setQuery(std::string queryStr)
{
	if(queryStr.length() == 0)
		return false;

	{
		bool						between = false;
		std::string					keyVal, val;
		std::vector< std::string >	queryList;

		queryData.clear();
		queryList = string_token(queryStr, " =");

		/*
		 * queryListの内容は、多分以下のような状態になっている
		 * [0] : フィールド名
		 * [1] : 値
		 * [2] : AND or and
		 * [3] : フィールド名
		 * [4] : 値
		 * [5] : AND or and
		 * [6] : フィールド名
		 * [7] : between or BETWEEN
		 * [8] : 値
		 * [9] : AND or and
		 * [10] : 値
		 */
		keyVal	= "";
		val		= "";
		for(std::vector< std::string >::iterator it = queryList.begin(); it != queryList.end(); it++){
			if( (*it == "AND") || (*it == "and") || (*it == "And") ){
				if(between == false)
					continue;
				else{
					val = val + ",";
					continue;
				}
			}

			if( (*it == "BETWEEN") || (*it == "between") || (*it == "Between") ){
				between = true;
//				isDateString(*it)
				continue;
			}

			if(keyVal.length() == 0)
				keyVal = *it;
			else{
				if(val.length() == 0)
					val = *it;
				else{
					val = val + *it;
					between = false;
				}
			}

			if( (( keyVal.length() > 0 ) && ( val.length() > 0 ) ) && (between == false)){
				queryData[keyVal] = val;
/*
std::cout << keyVal << " / " << val << std::endl;
*/
				keyVal	= "";
				val		= "";
			}
		}

		return true;
	}
}

std::string TextDataFile::getData(const std::string key, const std::string fieldName)
{
	return recordBuff[key][fieldName];
}

field_value TextDataFile::getFields()
{
	field_value result;

	for(field_list_it fit = fieldNameList.begin(); fit!= fieldNameList.end(); fit++){
		// フィールド分回る
		result.push_back((*fit).fieldName);
	}

	return result;
}

field_value TextDataFile::getRecord()
{
	field_value	result;

	for(field_list_it it = fieldNameList.begin(); it != fieldNameList.end(); it++){
		result.push_back((curRecord->second)[(*it).fieldName]);
	}

	return result;
}

record_it TextDataFile::getTopRecord()
{
	curRecord = recordBuff.begin();

	curRecord = CheckFilter(curRecord);

	return curRecord;
}

record_it TextDataFile::getNextRecord()
{
	if(curRecord == (record_it)NULL){
		return getTopRecord();
	}

	curRecord++;

	curRecord = CheckFilter(curRecord);

	if(curRecord == recordBuff.end())
		curRecord = (record_it)NULL;

	return curRecord;
}

record_it TextDataFile::CheckFilter(record_it it)
{
	if(queryData.size() > 0){
		bool	bMatch = true;

		for(; curRecord != recordBuff.end(); curRecord++){
			for(field_it it = queryData.begin(); it != queryData.end(); it++){
				if(isRange(it->first) == false){
					if( ((curRecord->second)[it->first]).find(it->second) != std::string::npos){
						bMatch = true;
//						return curRecord;
					}
					else{
						bMatch = false;
						break;
					}
				}
				else{
					// 範囲を取り出す
					std::vector< std::string >	rangeList;
					unsigned long dateVal, filterS, filterE;

					/*
					 *  日付文字列を整数に変換
					 *  2009/3/10 -> 20090310
					 */
					rangeList = string_token(it->second, ",");

					if(rangeList.size() == 2){
						filterS = isDateString((curRecord->second)[it->first])	?
									DateStrToULong(rangeList[0])				:
									string_to_int((curRecord->second)[it->first]);

						filterE = isDateString((curRecord->second)[it->first])	?
									DateStrToULong(rangeList[1])				:
									string_to_int((curRecord->second)[it->first]);
					}
					else{
						if(it->second[0] == ','){
							// 終了のみ指定されている
							filterS = 0;
							filterE = isDateString((curRecord->second)[it->first])	?
										DateStrToULong(rangeList[0])				:
										string_to_int((curRecord->second)[it->first]);
						}
						else{
							// 開始のみ指定されている
							filterS = isDateString((curRecord->second)[it->first])	?
										DateStrToULong(rangeList[0])				:
										string_to_int((curRecord->second)[it->first]);
							filterE = (unsigned long)(-1);
						}
					}

					dateVal = isDateString((curRecord->second)[it->first]) ?
							DateStrToULong((curRecord->second)[it->first]) :
							string_to_int((curRecord->second)[it->first]);
/*
std::cout << "Data = " << (curRecord->second)[it->first] << " / " << std::endl;
std::cout << "Flag = " << isDateString((curRecord->second)[it->first]) << " / " << std::endl;
std::cout << "開始日 = " << filterS << std::endl;
std::cout << " / 終了日 = " << filterE << std::endl;
std::cout << " / 比較 = " << dateVal << std::endl;
*/
					if( (dateVal >= filterS) && (dateVal <= filterE)) {
						bMatch = true;
					}
					else{
						bMatch = false;
					}
				}
			}
			if(bMatch)
				return curRecord;
		}

		return recordBuff.end();
	}

	return curRecord;
}

/*
str_it TextDataFile::getTopField()
{
	curField = fieldNameList.begin();

	return curField;
}

str_it TextDataFile::getNextField()
{
	curField++;

	return curField;
}
*/

void FieldData::setFieldName(std::string value)
{
	std::vector<std::string>	fieldData;

	fieldName_org	= value;
	fieldName		= value;

	fieldData = string_token(value, "{,}");
	/*
	 * 上記処理で、フィールド名・属性・型　に分解されたはず
	 */
	if(fieldData.size() == 3){
		fieldName = fieldData[0];
		if(fieldData[1].compare("Serial") == 0){
			// シリアル値
			fieldType = ftSerial;
		}
		if(fieldData[1].compare("Int") == 0){
			// 数値
			fieldType = ftInteger;
		}
		if(fieldData[1].compare("Date") == 0){
			// 日付
			fieldType = ftDate;
		}
		if(fieldData[1].compare("Time") == 0){
			// 時刻
			fieldType = ftTime;
		}
		if(fieldData[1].compare("Str") == 0){
			// 文字列
			fieldType = ftString;
		}

		for(int cnt = 0; cnt < (int)(fieldData[2].length()); cnt++){
			switch(fieldData[2][cnt]){
			case '~':	// 範囲指定可能
				rangeFilter = true;
				break;
			case '+':	// 追加のみ可能
				modFlg		= false;
				addFlg		= true;
				break;
			case '*':	// 全て可能
				modFlg		= true;
				addFlg		= true;
				break;
			case '-':	// 変更・追加不可能
			default:
				modFlg		= false;
				addFlg		= false;
				break;
			}
		}
	}

	return;
}
