/*
 * TextData.h
 *
 *  Created on: 2010/12/10
 *      Author: yotubata
 */

#ifndef TEXTDATA_H_
#define TEXTDATA_H_

#include <string>
#include <vector>
#include <map>
#include <fstream>


typedef enum _FieldType{
	ftSerial,
	ftInteger,
	ftDate,
	ftTime,
	ftString
} FieldType;

class FieldData {
public:
	FieldData()	{ fieldName_org = ""; fieldName = ""; modFlg = true; rangeFilter = false; }

	std::string		fieldName_org;
	std::string		fieldName;
	bool			modFlg;
	bool			addFlg;
	bool			rangeFilter;
	FieldType		fieldType;

public:
	void setFieldName(std::string fieldName);

public:
	operator std::string()	{ return fieldName; }
};

typedef std::vector<std::string>				field_value;
typedef field_value::iterator					str_it;

typedef std::vector<FieldData>					field_list;
typedef field_list::iterator					field_list_it;

typedef std::map< std::string, std::string>	field_data;
typedef field_data::value_type					field_pair;
typedef field_data::iterator					field_it;

typedef std::map< std::string, field_data >	record_data;
typedef record_data::value_type				record_pair;
typedef record_data::iterator					record_it;


class TextDataFile
{
public:
	TextDataFile(const std::string fileName = "", const bool writeMode = false, const unsigned long maxRecord = 0);
//	~TextDataFile();

public:
	virtual bool		openFile(const std::string fileName = "");
	virtual bool		createFile(const std::string fileName, const field_value dataFieldName);
	virtual bool		createFile(const std::string fileName, const field_list dataFieldName);
	virtual int			readData();
	virtual int			writeData(const std::string fileName = "");
	virtual bool		writeRecord(const std::string key, const field_value recordData);
	virtual bool		removeRecord(const std::string key);
	virtual std::string getData(const std::string key, const std::string fieldName);
	virtual bool		addField(std::string fieldName);
	virtual bool		setQuery(std::string queryStr);

	virtual record_it	getTopRecord();
	virtual record_it	getNextRecord();
/*
	str_it		getTopField();
	str_it		getNextField();
*/
	virtual field_value	getFields();
	virtual field_value	getRecord();

	virtual bool		isAdd(std::string	fieldName);
	virtual bool		isModify(std::string	fieldName);
	virtual bool		isRange(std::string	fieldName);
	virtual bool		isDate(std::string	fieldName);
	virtual bool		isTime(std::string	fieldName);

private:
	bool writeFields();
	bool writeRecord();

	void copyFieldData(/*const */field_value fieldVal, field_list fieldList);
	FieldData FindFieldData(std::string fieldName);
	record_it CheckFilter(record_it it);

public:
	field_data &operator[](std::string key);

private:
	std::string		dataFileName;
	std::ifstream	ifsDataFile;
	std::ofstream	ofsDataFile;
	unsigned long	buffSize;
	bool			allReaded;
	record_it		curRecord;
	str_it			curField;
	field_data		dummy_data;
	field_data		queryData;

protected:
	record_data		recordBuff;
	field_list		fieldNameList;
};

#endif /* TEXTDATA_H_ */
